package object;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;

import stage.MainStage;
import densan.s.game.drawing.Drawer;
import densan.s.game.input.KeyInput;
import densan.s.game.manager.GameManager;
import densan.s.game.object.GameRectBase;

public class Racket extends GameRectBase{
	
	// ラケットのサイズ
    public static final int WIDTH = 80;
    public static final int HEIGHT = 10;
    
    public static final int X = (GameManager.getInstance().getFrameWidth() - WIDTH) / 3;
    public static final int Y = GameManager.getInstance().getFrameHeight() - 10*HEIGHT;
    
 // ボールの当たり位置
    public static final int NO_COLLISION = 0;
    public static final int DOWN = 1;
    public static final int LEFT = 2;
    public static final int RIGHT = 3;
    public static final int UP = 4;
    public static final int DOWN_LEFT = 5;
    public static final int DOWN_RIGHT = 6;
    public static final int UP_LEFT = 7;
    public static final int UP_RIGHT = 8;
    
    public boolean isFiring;

    public Racket() {
    	//ラケットの座標,サイズを指定
    	super(X,Y,WIDTH,HEIGHT);    
    	isFiring = false;
    }
    
    public void update(){
    	if(KeyInput.isPressing(KeyEvent.VK_LEFT)&&getX()>0)
    		addX(-5);
    	else if(KeyInput.isPressing(KeyEvent.VK_RIGHT)&&getX()+WIDTH<GameManager.getInstance().getFrameWidth())
    		addX(5);
    }
    
    public int collideWith(Ball ball) {
        Rectangle racketRect = new Rectangle((int)getX(), (int)getY(), WIDTH, HEIGHT);

        int ballX = (int)ball.getX();
        int ballY = (int)ball.getY();
        int ballSize = ball.getHeight();
        
        if (racketRect.contains(ballX, ballY)
            && racketRect.contains(ballX + ballSize, ballY)) {
            // ブロックの下から衝突＝ボールの左上・右上の点がブロック内
            return DOWN;
        } else if (racketRect.contains(ballX + ballSize, ballY)
            && racketRect.contains(ballX + ballSize, ballY + ballSize)) {
            // ブロックの左から衝突＝ボールの右上・右下の点がブロック内
            return LEFT;
        } else if (racketRect.contains(ballX, ballY)
            && racketRect.contains(ballX, ballY + ballSize)) {
            // ブロックの右から衝突＝ボールの左上・左下の点がブロック内
            return RIGHT;
        } else if (racketRect.contains(ballX, ballY + ballSize)
            && racketRect.contains(ballX + ballSize, ballY + ballSize)) {
            // ブロックの上から衝突＝ボールの左下・右下の点がブロック内
            return UP;
        } else if (racketRect.contains(ballX + ballSize, ballY)) {
            // ブロックの左下から衝突＝ボールの右上の点がブロック内
            return DOWN_LEFT;
        } else if (racketRect.contains(ballX, ballY)) {
            // ブロックの右下から衝突＝ボールの左上の点がブロック内
            return DOWN_RIGHT;
        } else if (racketRect.contains(ballX + ballSize, ballY + ballSize)) {
            // ブロックの左上から衝突＝ボールの右下の点がブロック内
            return UP_LEFT;
        } else if (racketRect.contains(ballX, ballY + ballSize)) {
            // ブロックの右上から衝突＝ボールの左下の点がブロック内
            return UP_RIGHT;
        }

        return NO_COLLISION;
    }
    
    public void firing(){
    	isFiring = true;
    }
    
    public void reset(){
    	isFiring = false;
    }
    
    public void draw(Drawer d){
    	//色塗り
    	d.setColor(Color.RED);
		d.fillRect((int)getX(), (int)getY(), getWidth(), getHeight());
		
		 // 枠線を描画
        d.setColor(Color.BLACK);
        d.drawRect((int)getX(), (int)getY(), getWidth(), getHeight());
    }
    
    
}
