package stage;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.KeyEvent;

import main.ClearScene;
import main.ContinueScene;
import main.Main;
import main.Scene;
import densan.s.game.drawing.Drawer;
import densan.s.game.drawing.DrawingTool;
import densan.s.game.input.KeyInput;
import densan.s.game.manager.GameManager;
import object.Ball;
import object.Block;
import object.Racket;

public abstract class MainStage implements Scene{
	
	protected Racket racket;
	protected Ball ball;
	
	//フレーム countはupdate一周で+1
	private int count = 0;
	
	//60フレーム
	private static final int START_TIME = 60;
	
	public int stageNumber;
	
	//残機
	private int remainingLives = 3;
	
	//情報板の幅
	public static final int BOARD_WIDTH = 200;
	
	
    protected Block[] block; // ブロック
    
    //ブロックが消えた回数
	private int deadCount = 0;
	
	//ブロックの数
	private  int numBlock;
	
	
	/**
	 * 
	 * @param numBlock ブロックの数
	 * @param stageNumber ステージ番号
	 */
	public MainStage(int numBlock,int stageNumber){
		racket = new Racket();
		ball = new Ball(Racket.X+Racket.WIDTH/2);
		this.numBlock = numBlock;
		this.stageNumber = stageNumber;
		block = new Block[numBlock];
	}
	
	//updateでゲームを処理している
	@Override
	public void update() {
		
		//60フレーム後ラケットの移動が可能
		if(count>START_TIME){
		//ラケットの移動
			racket.update();
	
		//ボールの移動
			if(racket.isFiring)
				ball.update();
			
			//ラケットに合わせてボールが動く
			else ball.setX(racket.getX()+Racket.WIDTH/2-ball.getWidth()/2);
			
		//ボールの発射
			if(KeyInput.isPress(KeyEvent.VK_SPACE))
				racket.firing();
		}
		
		//attackPosは当たった位置　collideWithは当たり判定
        int attackPos = racket.collideWith(ball);
            // ボールの当たった位置からボールの反射方向を計算
            switch (attackPos) {
                case Racket.DOWN :
                case Racket.UP :
                    ball.boundY();
                    break;
                case Racket.LEFT :
                case Racket.RIGHT :
                    ball.boundX();
                    break;
                case Racket.UP_LEFT :
                case Racket.UP_RIGHT :
                case Racket.DOWN_LEFT :
                case Racket.DOWN_RIGHT :
                    ball.boundXY();
                    break;
            }
           
		
		
		// ブロックとボールの衝突処理
        for (int i = 0; i < numBlock; i++) {
            // すでに消えているブロックは無視
            if (block[i].isDead)
                continue;
            // ブロックの当たった位置を計算
            int collidePos = block[i].collideWith(ball);
            // ブロックに当たっていたら
            if (collidePos != Block.NO_COLLISION) {
                block[i].dead();
                deadCount += 1;
                // ボールの当たった位置からボールの反射方向を計算
                switch (collidePos) {
                    case Block.DOWN :
                    case Block.UP :
                        ball.boundY();
                        break;
                    case Block.LEFT :
                    case Block.RIGHT :
                        ball.boundX();
                        break;
                }
                break; // 1回に壊せるブロックは1つ
            }
        }
        
        // 画面外にでたら残機-1してボールを新しくセット
       if(ball.getY() > GameManager.getInstance().getFrameHeight()){
    	   remainingLives -= 1;
       		ball = new Ball(racket.getX()+Racket.WIDTH/2);
       		racket.reset();
        	//この部分に残機が0になった場合の処理を追加
       		
       		
       		
       		
       }
       
       //この部分にステージをクリアした場合の処理を追加
       
       
       
       
       
       
       
        count++;
	}
	
	public abstract MainStage nowStage();
	public abstract MainStage newStage();
	public abstract MainStage nextStage();

	@Override
	public void draw(Drawer d) {
		//ラケットの描画
		racket.draw(d);
		//ボールの描画
		ball.draw(d);
		
		// ブロックが死んでなかったらブロックを描画
		for (int i = 0; i < numBlock; i++)
				if(!block[i].isDead)
					block[i].draw(d); 
		
		//この部分にボード表の処理を追加
		
		
		
		
		
		
		
		
		
		
		
			
		//最初にステージ名を表示
		if(count<=START_TIME){
			d.setColor(Color.RED);
			d.setFont(new Font(Font.DIALOG,Font.ITALIC,30));
			d.drawStringCenter("STAGE"+stageNumber, GameManager.getInstance().getFrameWidth()/3+3,
								GameManager.getInstance().getFrameHeight()/2+3);
			d.setColor(Color.BLACK);
			d.setFont(new Font(Font.DIALOG,Font.ITALIC,30));
			d.drawStringCenter("STAGE"+stageNumber, GameManager.getInstance().getFrameWidth()/3,
								GameManager.getInstance().getFrameHeight()/2);
		}
	}
}
